diff -ru nginx-0.8.40.orig/src/http/modules/ngx_http_fastcgi_module.c nginx-0.8.40/src/http/modules/ngx_http_fastcgi_module.c
--- nginx-0.8.40.orig/src/http/modules/ngx_http_fastcgi_module.c	2010-06-04 11:17:09.000000000 +0200
+++ nginx-0.8.40/src/http/modules/ngx_http_fastcgi_module.c	2010-06-09 23:40:05.000000000 +0200
@@ -425,6 +425,22 @@
       offsetof(ngx_http_fastcgi_loc_conf_t, upstream.hide_headers),
       NULL },
 
+#if (NGX_HTTP_CACHE)
+    { ngx_string("fastcgi_pass_cache_header"),
+      NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_FLAG,
+      ngx_conf_set_str_array_slot,
+      NGX_HTTP_LOC_CONF_OFFSET,
+      offsetof(ngx_http_fastcgi_loc_conf_t, upstream.pass_cache_headers),
+      NULL },
+
+    { ngx_string("fastcgi_hide_cache_header"),
+      NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_FLAG,
+      ngx_conf_set_str_array_slot,
+      NGX_HTTP_LOC_CONF_OFFSET,
+      offsetof(ngx_http_fastcgi_loc_conf_t, upstream.hide_cache_headers),
+      NULL },
+#endif
+
     { ngx_string("fastcgi_ignore_headers"),
       NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_1MORE,
       ngx_conf_set_bitmask_slot,
@@ -525,12 +541,6 @@
 #if (NGX_HTTP_CACHE)
 
 static ngx_str_t  ngx_http_fastcgi_hide_cache_headers[] = {
-    ngx_string("Status"),
-    ngx_string("X-Accel-Expires"),
-    ngx_string("X-Accel-Redirect"),
-    ngx_string("X-Accel-Limit-Rate"),
-    ngx_string("X-Accel-Buffering"),
-    ngx_string("X-Accel-Charset"),
     ngx_string("Set-Cookie"),
     ngx_string("P3P"),
     ngx_null_string
@@ -1975,6 +1985,7 @@
      *     conf->upstream.cache_methods = 0;
      *     conf->upstream.temp_path = NULL;
      *     conf->upstream.hide_headers_hash = { NULL, 0 };
+     *     conf->upstream.hide_cache_headers_hash = { NULL, 0 };
      *     conf->upstream.uri = { 0, NULL };
      *     conf->upstream.location = NULL;
      *     conf->upstream.store_lengths = NULL;
@@ -2012,6 +2023,11 @@
     conf->upstream.hide_headers = NGX_CONF_UNSET_PTR;
     conf->upstream.pass_headers = NGX_CONF_UNSET_PTR;
 
+#if (NGX_HTTP_CACHE)
+    conf->upstream.hide_cache_headers = NGX_CONF_UNSET_PTR;
+    conf->upstream.pass_cache_headers = NGX_CONF_UNSET_PTR;
+#endif
+
     conf->upstream.intercept_errors = NGX_CONF_UNSET;
 
     /* "fastcgi_cyclic_temp_file" is disabled */
@@ -2032,12 +2048,14 @@
     u_char                       *p;
     size_t                        size;
     uintptr_t                    *code;
-    ngx_str_t                    *hide;
     ngx_uint_t                    i;
     ngx_array_t                   headers_names;
     ngx_keyval_t                 *src;
     ngx_hash_key_t               *hk;
     ngx_hash_init_t               hash;
+#if (NGX_HTTP_CACHE)
+    ngx_hash_init_t               cache_hash;
+#endif
     ngx_http_script_compile_t     sc;
     ngx_http_script_copy_code_t  *copy;
 
@@ -2259,21 +2277,29 @@
     hash.name = "fastcgi_hide_headers_hash";
 
 #if (NGX_HTTP_CACHE)
-
-    hide = conf->upstream.cache ? ngx_http_fastcgi_hide_cache_headers:
-                                  ngx_http_fastcgi_hide_headers;
-#else
-
-    hide = ngx_http_fastcgi_hide_headers;
-
+    cache_hash.max_size = 512;
+    cache_hash.bucket_size = ngx_align(64, ngx_cacheline_size);
+    cache_hash.name = "fastcgi_hide_cache_headers_hash";
 #endif
 
     if (ngx_http_upstream_hide_headers_hash(cf, &conf->upstream,
-                                            &prev->upstream, hide, &hash)
+                                            &prev->upstream,
+                                            ngx_http_fastcgi_hide_headers, &hash)
+        != NGX_OK)
+    {
+        return NGX_CONF_ERROR;
+    }
+
+#if (NGX_HTTP_CACHE)
+    if (ngx_http_upstream_hide_cache_headers_hash(cf, &conf->upstream,
+                                            &prev->upstream,
+                                            ngx_http_fastcgi_hide_cache_headers,
+                                            &cache_hash)
         != NGX_OK)
     {
         return NGX_CONF_ERROR;
     }
+#endif
 
     if (conf->upstream.upstream == NULL) {
         conf->upstream.upstream = prev->upstream.upstream;
Seulement dans nginx-0.8.40/src/http/modules: ngx_http_fastcgi_module.c.orig
Seulement dans nginx-0.8.40/src/http/modules: ngx_http_fastcgi_module.c.rej
diff -ru nginx-0.8.40.orig/src/http/modules/ngx_http_proxy_module.c nginx-0.8.40/src/http/modules/ngx_http_proxy_module.c
--- nginx-0.8.40.orig/src/http/modules/ngx_http_proxy_module.c	2010-06-03 16:50:59.000000000 +0200
+++ nginx-0.8.40/src/http/modules/ngx_http_proxy_module.c	2010-06-09 23:40:44.000000000 +0200
@@ -437,6 +437,22 @@
       offsetof(ngx_http_proxy_loc_conf_t, upstream.hide_headers),
       NULL },
 
+#if (NGX_HTTP_CACHE)
+    { ngx_string("proxy_pass_cache_header"),
+      NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_FLAG,
+      ngx_conf_set_str_array_slot,
+      NGX_HTTP_LOC_CONF_OFFSET,
+      offsetof(ngx_http_proxy_loc_conf_t, upstream.pass_cache_headers),
+      NULL },
+
+    { ngx_string("proxy_hide_cache_header"),
+      NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_FLAG,
+      ngx_conf_set_str_array_slot,
+      NGX_HTTP_LOC_CONF_OFFSET,
+      offsetof(ngx_http_proxy_loc_conf_t, upstream.hide_cache_headers),
+      NULL },
+#endif
+
     { ngx_string("proxy_ignore_headers"),
       NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_1MORE,
       ngx_conf_set_bitmask_slot,
@@ -533,14 +549,6 @@
 
 
 static ngx_str_t  ngx_http_proxy_hide_cache_headers[] = {
-    ngx_string("Date"),
-    ngx_string("Server"),
-    ngx_string("X-Pad"),
-    ngx_string("X-Accel-Expires"),
-    ngx_string("X-Accel-Redirect"),
-    ngx_string("X-Accel-Limit-Rate"),
-    ngx_string("X-Accel-Buffering"),
-    ngx_string("X-Accel-Charset"),
     ngx_string("Set-Cookie"),
     ngx_string("P3P"),
     ngx_null_string
@@ -1885,6 +1893,7 @@
      *     conf->upstream.cache_methods = 0;
      *     conf->upstream.temp_path = NULL;
      *     conf->upstream.hide_headers_hash = { NULL, 0 };
+     *     conf->upstream.hide_cache_headers_hash = { NULL, 0 };
      *     conf->upstream.uri = { 0, NULL };
      *     conf->upstream.location = NULL;
      *     conf->upstream.store_lengths = NULL;
@@ -1930,6 +1939,11 @@
     conf->upstream.hide_headers = NGX_CONF_UNSET_PTR;
     conf->upstream.pass_headers = NGX_CONF_UNSET_PTR;
 
+#if (NGX_HTTP_CACHE)
+    conf->upstream.hide_cache_headers = NGX_CONF_UNSET_PTR;
+    conf->upstream.pass_cache_headers = NGX_CONF_UNSET_PTR;
+#endif
+
     conf->upstream.intercept_errors = NGX_CONF_UNSET;
 #if (NGX_HTTP_SSL)
     conf->upstream.ssl_session_reuse = NGX_CONF_UNSET;
@@ -1955,9 +1969,11 @@
     ngx_http_proxy_loc_conf_t *conf = child;
 
     size_t                      size;
-    ngx_str_t                  *h;
     ngx_keyval_t               *s;
     ngx_hash_init_t             hash;
+#if (NGX_HTTP_CACHE)
+    ngx_hash_init_t             cache_hash;
+#endif
     ngx_http_proxy_redirect_t  *pr;
     ngx_http_script_compile_t   sc;
 
@@ -2240,21 +2256,29 @@
     hash.name = "proxy_headers_hash";
 
 #if (NGX_HTTP_CACHE)
-
-    h = conf->upstream.cache ? ngx_http_proxy_hide_cache_headers:
-                               ngx_http_proxy_hide_headers;
-#else
-
-    h = ngx_http_proxy_hide_headers;
-
+    cache_hash.max_size = conf->headers_hash_max_size;
+    cache_hash.bucket_size = conf->headers_hash_bucket_size;
+    cache_hash.name = "proxy_cache_headers_hash";
 #endif
 
     if (ngx_http_upstream_hide_headers_hash(cf, &conf->upstream,
-                                            &prev->upstream, h, &hash)
+                                            &prev->upstream,
+                                            ngx_http_proxy_hide_headers, &hash)
+        != NGX_OK)
+    {
+        return NGX_CONF_ERROR;
+    }
+
+#if (NGX_HTTP_CACHE)
+    if (ngx_http_upstream_hide_cache_headers_hash(cf, &conf->upstream,
+                                            &prev->upstream,
+                                            ngx_http_proxy_hide_cache_headers,
+                                            &cache_hash)
         != NGX_OK)
     {
         return NGX_CONF_ERROR;
     }
+#endif
 
     if (conf->upstream.upstream == NULL) {
         conf->upstream.upstream = prev->upstream.upstream;
Seulement dans nginx-0.8.40/src/http/modules: ngx_http_proxy_module.c.orig
diff -ru nginx-0.8.40.orig/src/http/modules/ngx_http_uwsgi_module.c nginx-0.8.40/src/http/modules/ngx_http_uwsgi_module.c
--- nginx-0.8.40.orig/src/http/modules/ngx_http_uwsgi_module.c	2010-06-04 14:55:01.000000000 +0200
+++ nginx-0.8.40/src/http/modules/ngx_http_uwsgi_module.c	2010-06-09 23:41:26.000000000 +0200
@@ -338,6 +338,22 @@
       offsetof(ngx_http_uwsgi_loc_conf_t, upstream.hide_headers),
       NULL },
 
+#if (NGX_HTTP_CACHE)
+    { ngx_string("uwsgi_pass_cache_header"),
+      NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_FLAG,
+      ngx_conf_set_str_array_slot,
+      NGX_HTTP_LOC_CONF_OFFSET,
+      offsetof(ngx_http_uwsgi_loc_conf_t, upstream.pass_cache_headers),
+      NULL },
+
+    { ngx_string("uwsgi_hide_cache_header"),
+      NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_FLAG,
+      ngx_conf_set_str_array_slot,
+      NGX_HTTP_LOC_CONF_OFFSET,
+      offsetof(ngx_http_uwsgi_loc_conf_t, upstream.hide_cache_headers),
+      NULL },
+#endif
+
     { ngx_string("uwsgi_ignore_headers"),
       NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_1MORE,
       ngx_conf_set_bitmask_slot,
@@ -393,11 +409,6 @@
 #if (NGX_HTTP_CACHE)
 
 static ngx_str_t  ngx_http_uwsgi_hide_cache_headers[] = {
-    ngx_string("X-Accel-Expires"),
-    ngx_string("X-Accel-Redirect"),
-    ngx_string("X-Accel-Limit-Rate"),
-    ngx_string("X-Accel-Buffering"),
-    ngx_string("X-Accel-Charset"),
     ngx_string("Set-Cookie"),
     ngx_string("P3P"),
     ngx_null_string
@@ -1309,6 +1320,11 @@
     conf->upstream.hide_headers = NGX_CONF_UNSET_PTR;
     conf->upstream.pass_headers = NGX_CONF_UNSET_PTR;
 
+#if (NGX_HTTP_CACHE)
+    conf->upstream.hide_cache_headers = NGX_CONF_UNSET_PTR;
+    conf->upstream.pass_cache_headers = NGX_CONF_UNSET_PTR;
+#endif
+
     conf->upstream.intercept_errors = NGX_CONF_UNSET;
 
     /* "uwsgi_cyclic_temp_file" is disabled */
@@ -1327,12 +1343,14 @@
     u_char                       *p;
     size_t                        size;
     uintptr_t                    *code;
-    ngx_str_t                    *hide;
     ngx_uint_t                    i;
     ngx_array_t                   headers_names;
     ngx_keyval_t                 *src;
     ngx_hash_key_t               *hk;
     ngx_hash_init_t               hash;
+#if (NGX_HTTP_CACHE)
+    ngx_hash_init_t               cache_hash;
+#endif
     ngx_http_script_compile_t     sc;
     ngx_http_script_copy_code_t  *copy;
 
@@ -1549,22 +1567,30 @@
     hash.name = "uwsgi_hide_headers_hash";
 
 #if (NGX_HTTP_CACHE)
-
-    hide = conf->upstream.cache ? ngx_http_uwsgi_hide_cache_headers:
-                                  ngx_http_uwsgi_hide_headers;
-#else
-
-    hide = ngx_http_uwsgi_hide_headers;
-
+    cache_hash.max_size = 512;
+    cache_hash.bucket_size = ngx_align(64, ngx_cacheline_size);
+    cache_hash.name = "uwsgi_hide_cache_headers_hash";
 #endif
 
     if (ngx_http_upstream_hide_headers_hash(cf, &conf->upstream,
-                                            &prev->upstream, hide, &hash)
+                                            &prev->upstream,
+                                            ngx_http_uwsgi_hide_headers, &hash)
         != NGX_OK)
     {
         return NGX_CONF_ERROR;
     }
 
+#if (NGX_HTTP_CACHE)
+    if (ngx_http_upstream_hide_cache_headers_hash(cf, &conf->upstream,
+                                            &prev->upstream,
+                                            ngx_http_uwsgi_hide_cache_headers,
+                                            &cache_hash)
+        != NGX_OK)
+    {
+        return NGX_CONF_ERROR;
+    }
+#endif
+
     if (conf->upstream.upstream == NULL) {
         conf->upstream.upstream = prev->upstream.upstream;
     }
diff -ru nginx-0.8.40.orig/src/http/ngx_http_upstream.c nginx-0.8.40/src/http/ngx_http_upstream.c
--- nginx-0.8.40.orig/src/http/ngx_http_upstream.c	2010-05-24 14:35:10.000000000 +0200
+++ nginx-0.8.40/src/http/ngx_http_upstream.c	2010-06-09 23:43:52.000000000 +0200
@@ -1847,6 +1847,16 @@
             continue;
         }
 
+#if (NGX_HTTP_CACHE)
+        if (u->cacheable) {
+            if (ngx_hash_find(&u->conf->hide_cache_headers_hash, h[i].hash,
+                              h[i].lowcase_key, h[i].key.len))
+            {
+                continue;
+            }
+        }
+#endif
+
         hh = ngx_hash_find(&umcf->headers_in_hash, h[i].hash,
                            h[i].lowcase_key, h[i].key.len);
 
@@ -4354,6 +4364,118 @@
 }
 
 
+#if (NGX_HTTP_CACHE)
+ngx_int_t
+ngx_http_upstream_hide_cache_headers_hash(ngx_conf_t *cf,
+    ngx_http_upstream_conf_t *conf, ngx_http_upstream_conf_t *prev,
+    ngx_str_t *default_hide_cache_headers, ngx_hash_init_t *hash)
+{
+    ngx_str_t       *h;
+    ngx_uint_t       i, j;
+    ngx_array_t      hide_cache_headers;
+    ngx_hash_key_t  *hk;
+
+    if (conf->hide_cache_headers == NGX_CONF_UNSET_PTR
+        && conf->pass_cache_headers == NGX_CONF_UNSET_PTR)
+    {
+        conf->hide_cache_headers_hash = prev->hide_cache_headers_hash;
+
+        if (conf->hide_cache_headers_hash.buckets
+            && ((conf->cache == NULL) == (prev->cache == NULL))
+           )
+        {
+            return NGX_OK;
+        }
+
+        conf->hide_cache_headers = prev->hide_cache_headers;
+        conf->pass_cache_headers = prev->pass_cache_headers;
+
+    } else {
+        if (conf->hide_cache_headers == NGX_CONF_UNSET_PTR) {
+            conf->hide_cache_headers = prev->hide_cache_headers;
+        }
+
+        if (conf->pass_cache_headers == NGX_CONF_UNSET_PTR) {
+            conf->pass_cache_headers = prev->pass_cache_headers;
+        }
+    }
+
+    if (ngx_array_init(&hide_cache_headers, cf->temp_pool, 4, sizeof(ngx_hash_key_t))
+        != NGX_OK)
+    {
+        return NGX_ERROR;
+    }
+
+    for (h = default_hide_cache_headers; h->len; h++) {
+        hk = ngx_array_push(&hide_cache_headers);
+        if (hk == NULL) {
+            return NGX_ERROR;
+        }
+
+        hk->key = *h;
+        hk->key_hash = ngx_hash_key_lc(h->data, h->len);
+        hk->value = (void *) 1;
+    }
+
+    if (conf->hide_cache_headers != NGX_CONF_UNSET_PTR) {
+
+        h = conf->hide_cache_headers->elts;
+
+        for (i = 0; i < conf->hide_cache_headers->nelts; i++) {
+
+            hk = hide_cache_headers.elts;
+
+            for (j = 0; j < hide_cache_headers.nelts; j++) {
+                if (ngx_strcasecmp(h[i].data, hk[j].key.data) == 0) {
+                    goto exist;
+                }
+            }
+
+            hk = ngx_array_push(&hide_cache_headers);
+            if (hk == NULL) {
+                return NGX_ERROR;
+            }
+
+            hk->key = h[i];
+            hk->key_hash = ngx_hash_key_lc(h[i].data, h[i].len);
+            hk->value = (void *) 1;
+
+        exist:
+
+            continue;
+        }
+    }
+
+    if (conf->pass_cache_headers != NGX_CONF_UNSET_PTR) {
+
+        h = conf->pass_cache_headers->elts;
+        hk = hide_cache_headers.elts;
+
+        for (i = 0; i < conf->pass_cache_headers->nelts; i++) {
+            for (j = 0; j < hide_cache_headers.nelts; j++) {
+
+                if (hk[j].key.data == NULL) {
+                    continue;
+                }
+
+                if (ngx_strcasecmp(h[i].data, hk[j].key.data) == 0) {
+                    hk[j].key.data = NULL;
+                    break;
+                }
+            }
+        }
+    }
+
+    hash->hash = &conf->hide_cache_headers_hash;
+    hash->key = ngx_hash_key_lc;
+    hash->pool = cf->pool;
+    hash->temp_pool = NULL;
+
+    return ngx_hash_init(hash, hide_cache_headers.elts, hide_cache_headers.nelts);
+}
+#endif
+
+
 static void *
 ngx_http_upstream_create_main_conf(ngx_conf_t *cf)
 {
diff -ru nginx-0.8.40.orig/src/http/ngx_http_upstream.h nginx-0.8.40/src/http/ngx_http_upstream.h
--- nginx-0.8.40.orig/src/http/ngx_http_upstream.h	2010-05-24 13:01:05.000000000 +0200
+++ nginx-0.8.40/src/http/ngx_http_upstream.h	2010-06-09 23:41:48.000000000 +0200
@@ -152,6 +152,12 @@
     ngx_array_t                     *hide_headers;
     ngx_array_t                     *pass_headers;
 
+#if (NGX_HTTP_CACHE)
+    ngx_hash_t                       hide_cache_headers_hash;
+    ngx_array_t                     *hide_cache_headers;
+    ngx_array_t                     *pass_cache_headers;
+#endif
+
     ngx_addr_t                      *local;
 
 #if (NGX_HTTP_CACHE)
@@ -329,6 +335,11 @@
 ngx_int_t ngx_http_upstream_hide_headers_hash(ngx_conf_t *cf,
     ngx_http_upstream_conf_t *conf, ngx_http_upstream_conf_t *prev,
     ngx_str_t *default_hide_headers, ngx_hash_init_t *hash);
+#if (NGX_HTTP_CACHE)
+ngx_int_t ngx_http_upstream_hide_cache_headers_hash(ngx_conf_t *cf,
+    ngx_http_upstream_conf_t *conf, ngx_http_upstream_conf_t *prev,
+    ngx_str_t *default_hide_cache_headers, ngx_hash_init_t *hash);
+#endif
 
 
 #define ngx_http_conf_upstream_srv_conf(uscf, module)                         \